package ru.sinitsynme.sellingmarket.rest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.sinitsynme.sellingmarket.entity.Advertisement;
import ru.sinitsynme.sellingmarket.entity.dto.AdvertisementDto;
import ru.sinitsynme.sellingmarket.exception.UnsupportedFileTypeException;
import ru.sinitsynme.sellingmarket.service.AdvertisementService;
import ru.sinitsynme.sellingmarket.service.impl.filter.AdvFilter;

import javax.validation.Valid;
import java.io.IOException;
import java.net.URLConnection;
import java.util.List;
import java.util.UUID;

@RestController
@RequestMapping("/advert")
public class AdvertisementController {

    private static final String VIDEO_CONTENT_TYPE = "video";
    private static final String IMAGE_CONTENT_TYPE = "image";

    private final AdvertisementService advertisementService;

    @Autowired
    public AdvertisementController(AdvertisementService advertisementService) {
        this.advertisementService = advertisementService;
    }

    private void checkIfFilesHaveCorrectExtension(MultipartFile[] files) throws IOException {
        for (MultipartFile multipartFile : files) {
            String mimeType = URLConnection.guessContentTypeFromName(multipartFile.getOriginalFilename());

            if (!mimeType.startsWith(IMAGE_CONTENT_TYPE) && !mimeType.startsWith(VIDEO_CONTENT_TYPE)) throw new
                    UnsupportedFileTypeException("Only video or image are allowed");
        }
    }

    @GetMapping("/get/{id}")
    public ResponseEntity<Advertisement> getAdvertisementById(@PathVariable UUID id) {
        return ResponseEntity.ok(advertisementService.getAdvertisement(id));
    }

    @GetMapping("/get/filter")
    public ResponseEntity<List<Advertisement>> getAdvertisementByFilter(@RequestParam @Valid AdvFilter filter) {
        return ResponseEntity.ok(advertisementService.getByFilter(filter));
    }

    @PostMapping()
    @Secured({"ROLE_ADMIN", "ROLE_USER", "ROLE_MODERATOR"})
    public ResponseEntity<Advertisement> uploadAdvertisement(@RequestPart MultipartFile[] files, @RequestPart @Valid AdvertisementDto dto) throws IOException {

        if(files != null) checkIfFilesHaveCorrectExtension(files);

        return ResponseEntity.ok(advertisementService.addAdvertisement(dto, files));
    }

    @DeleteMapping("/{id}")
    @Secured({"ROLE_ADMIN", "ROLE_USER"})
    public ResponseEntity<?> deleteAdvertisement(@PathVariable UUID id) {
        advertisementService.deleteAdvertisement(id);
        return ResponseEntity.ok().build();
    }

    @PatchMapping("/edit/{id}")
    @Secured({"ROLE_ADMIN", "ROLE_USER", "ROLE_MODERATOR"})
    public ResponseEntity<Advertisement> editAdvertisement(@RequestBody @Valid AdvertisementDto advertisement, @PathVariable UUID id) {
        return ResponseEntity.ok(advertisementService.editAdvertisement(advertisement, id));
    }

}
