package ru.sinitsynme.sellingmarket.service.impl;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import ru.sinitsynme.sellingmarket.entity.*;
import ru.sinitsynme.sellingmarket.entity.dto.AdvertisementDto;
import ru.sinitsynme.sellingmarket.exception.ForbiddenException;
import ru.sinitsynme.sellingmarket.exception.ResourceNotFoundException;
import ru.sinitsynme.sellingmarket.repository.AdvRepository;
import ru.sinitsynme.sellingmarket.service.*;
import ru.sinitsynme.sellingmarket.service.impl.filter.AdvFilter;

import javax.transaction.Transactional;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@Service
public class AdvertisementServiceImpl implements AdvertisementService {

    @Value("${upload.path}")
    private String fileUploadPath;

    private final AdvRepository advRepository;

    private final AdvCategoryService categoryService;
    private final AdvSubcategoryService subcategoryService;
    private final AdvTypeService typeService;

    private final UserService userService;


    public AdvertisementServiceImpl(AdvRepository advRepository, AdvCategoryService categoryService,
                                    AdvSubcategoryService subcategoryService, AdvTypeService typeService, UserService userService) {
        this.advRepository = advRepository;
        this.categoryService = categoryService;
        this.subcategoryService = subcategoryService;
        this.typeService = typeService;
        this.userService = userService;
    }

    @Override
    public List<Advertisement> getByFilter(AdvFilter filter) {
        return advRepository.findAll().stream().filter(matchesFiltering(filter)).collect(Collectors.toList());
    }

    @Override
    public Advertisement getAdvertisement(UUID id) {
        return advRepository.findById(id).orElseThrow(() ->
                new ResourceNotFoundException(String.format("Advertisement with ID = %s was not found", id)));
    }

    @Override
    @Transactional
    public Advertisement addAdvertisement(AdvertisementDto dto, MultipartFile[] multipartRequest) throws IOException {
        Advertisement advertisement = new Advertisement();
        fillAdvertisement(dto, advertisement);

        advertisement.setUser(userService.getPrincipalEntity().orElseThrow(() ->
                new ForbiddenException("Authentication is missing")));

        if(multipartRequest.length != 0){
            advertisement.setLinks(saveFiles(multipartRequest));
        }

        return advRepository.save(advertisement);
    }

    @Override
    public Advertisement editAdvertisement(AdvertisementDto dto, UUID id) {
        Advertisement advertisement = getAdvertisementSecured(id);

        fillAdvertisement(dto, advertisement);

        return advertisement;
    }


    @Override
    public void deleteAdvertisement(UUID id) {
        getAdvertisementSecured(id);

        advRepository.deleteById(id);
    }

    private Predicate<Advertisement> matchesFiltering(AdvFilter filter) {
        return adv -> adv.getAdvType().getName().equals(filter.getAdvSubcategory().trim()) &&
                adv.getAdvCategory().getCategoryName().equals(filter.getAdvCategory().trim()) &&
                adv.getAdvSubcategoryList().stream().anyMatch(it -> it.getSubcategoryName().equals(filter.getAdvSubcategory().trim()));
    }

    private Advertisement getAdvertisementSecured(UUID id) {
        Advertisement advertisement = advRepository.findById(id).orElseThrow(() ->
                new ResourceNotFoundException(String.format("Advertisement with ID = %s not exists", id)));

        User principal = userService.getPrincipalEntity().orElseThrow(() ->
                new ForbiddenException("Authentication is missing"));

        if (principal.getUserRole().getName().equals("ROLE_USER")) {
            if (!advertisement.getUser().getId().equals(principal.getId())) {
                throw new ForbiddenException(String.format("This is not user's (ID = %s) advertisement!", principal.getId()));
            }
        }
        return advertisement;
    }


    private void fillAdvertisement(AdvertisementDto dto, Advertisement advertisement) {

        String categoryName = dto.getCategory();
        String typeName = dto.getType();
        List<String> subcategoriesNames = dto.getSubcategories();

        if(categoryName != null && !categoryName.isEmpty()){
            AdvCategory category = categoryService.getByName(categoryName);
            advertisement.setAdvCategory(category);

        }

        if( typeName != null && ! typeName.isEmpty()) {

            AdvType type = typeService.getByName(typeName);
            advertisement.setAdvType(type);
        }

        if(subcategoriesNames != null){
            List<AdvSubcategory> advSubcategoryList = subcategoryService.getAllByNames(subcategoriesNames);
            advertisement.setAdvSubcategoryList(advSubcategoryList);
        }


        advertisement.setText(dto.getText());
        advertisement.setName(dto.getAdvertName());
        advertisement.setDate(new Date());
    }

    private List<Link> saveFiles(MultipartFile[] multipartFiles) throws IOException {

        List<Link> links = new ArrayList<>();

        for (MultipartFile multipartFile : multipartFiles){
            if (multipartFile != null) {
                File uploadDir = new File(fileUploadPath);

                if (!uploadDir.exists()) {
                    uploadDir.mkdir();
                }

                String uuidFile = UUID.randomUUID().toString();
                String resultFilename = uuidFile + "." + multipartFile.getOriginalFilename();

                multipartFile.transferTo(new File(fileUploadPath + "/" + resultFilename));

                links.add(new Link(resultFilename));
            }
        }

        return links;
    }
}
