package ru.sinitsynme.sellingmarket.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import ru.sinitsynme.sellingmarket.entity.User;
import ru.sinitsynme.sellingmarket.entity.UserRole;
import ru.sinitsynme.sellingmarket.entity.dto.UserDto;
import ru.sinitsynme.sellingmarket.exception.ForbiddenException;
import ru.sinitsynme.sellingmarket.exception.ResourceNotFoundException;
import ru.sinitsynme.sellingmarket.repository.UserRepository;
import ru.sinitsynme.sellingmarket.repository.UserRoleRepository;
import ru.sinitsynme.sellingmarket.service.UserService;

import java.nio.CharBuffer;
import java.util.Arrays;
import java.util.Optional;
import java.util.Set;
import java.util.UUID;

@Service
public class UserServiceImpl implements UserDetailsService, UserService {

    private final UserRepository userRepository;

    private final PasswordEncoder passwordEncoder;

    private final UserRoleRepository userRoleRepository;

    @Autowired
    public UserServiceImpl(UserRepository userRepository, PasswordEncoder passwordEncoder, UserRoleRepository userRoleRepository) {
        this.userRepository = userRepository;
        this.passwordEncoder = passwordEncoder;
        this.userRoleRepository = userRoleRepository;
    }

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        User user = userRepository.findByUsername(username).orElseThrow(() ->
                new UsernameNotFoundException(String.format("User with username %s was not found", username)));

        return new org.springframework.security.core.userdetails.User(user.getUsername(), user.getPassword(), Set.of(user.getUserRole()));
    }

    @Override
    public User getUser(UUID userId) {
        return userRepository.findById(userId).orElseThrow(() ->
                new ResourceNotFoundException(String.format("User with id %s was not found", userId)));
    }

    @Override
    public User addUser(UserDto dto) {
        User user = new User();
        user.setUsername(dto.getUsername());

        if(userRepository.findByUsername(dto.getUsername()).isPresent())
            throw new ForbiddenException("The user with the same username already exists");

        //hiding password from string pool using CharBuffer and char[] instead of string password

        CharBuffer charBuffer = CharBuffer.wrap(dto.getPassword());
        user.setPassword(passwordEncoder.encode(charBuffer));

        Arrays.fill(dto.getPassword(), '*');

        //adding ROLE_USER
        UserRole userRole = userRoleRepository.findByName("ROLE_USER").orElseThrow(()
                -> new ResourceNotFoundException("Some issues on serverside"));

        userRepository.save(user);
        user.setUserRole(userRole);

        return userRepository.save(user);
    }

    @Override
    public User editRole(String roleName, UUID userId) {
        User user = userRepository.findById(userId).orElseThrow(() -> new ResourceNotFoundException(String.format("User with ID = %s was not found", userId)));
        UserRole userRole = userRoleRepository.findByName(roleName).orElseThrow(() -> new ResourceNotFoundException(String.format("Role with name = %s was not found", roleName)));

        user.setUserRole(userRole);

        return userRepository.save(user);
    }

    @Override
    public User editName(String name, UUID userId) {
        User user = userRepository.findById(userId).orElseThrow(() -> new ResourceNotFoundException(String.format("User with ID = %s was not found", userId)));

        user.setUsername(name);
        return userRepository.save(user);
    }

    @Override
    public void deleteUser(UUID id) {
        throwExceptionIfNotExists(id);
        userRepository.deleteById(id);

    }

    @Override
    public void changeAdvertisementRights(UUID id, boolean enable) {
        User user = userRepository.findById(id).orElseThrow(() -> new ResourceNotFoundException(String.format("User with ID = %s was not found", id)));

        user.setAdvertisementPermission(enable);

        userRepository.save(user);
    }


    @Override
    public void throwExceptionIfNotExists(UUID id) {
        if (!userRepository.existsById(id))
            throw new ResourceNotFoundException(String.format("User with ID = %s wasn't found", id));
    }

    @Override
    public Optional<User> getPrincipalEntity() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();

        if (!(authentication instanceof AnonymousAuthenticationToken)) {
            String username = ((UserDetails) authentication.getPrincipal()).getUsername();
            return userRepository.findByUsername(username);
        }

        return Optional.empty();
    }

    @Override
    public boolean isPrincipalAvailable() {
        return getPrincipalEntity().isPresent();
    }
}
